import{a5 as e}from"./c.998383f4.js";const a="serial"in navigator,s=window.isSecureContext,P={ESP32:{label:"ESP32",showInPickerTitle:!0,defaultBoard:"esp32dev"},ESP32S2:{label:"ESP32-S2",showInPickerTitle:!0,defaultBoard:"esp32-s2-saola-1"},ESP32S3:{label:"ESP32-S3",showInPickerTitle:!0,defaultBoard:"esp32-s3-devkitc-1"},ESP32C3:{label:"ESP32-C3",showInPickerTitle:!0,defaultBoard:"esp32-c3-devkitm-1"},ESP8266:{label:"ESP8266",showInPickerTitle:!0,defaultBoard:"esp01_1m"},RP2040:{label:"Raspberry Pi Pico W",showInPickerTitle:!1,defaultBoard:"rpipicow"}},S={ESP32:"ESP32","ESP32-S2":"ESP32S2","ESP32-S3":"ESP32S3","ESP32-C3":"ESP32C3",ESP8266:"ESP8266"},i=e`
  <svg width="24" height="24" viewBox="0 0 24 24" slot="meta">
    <path d="M8.59,16.58L13.17,12L8.59,7.41L10,6L16,12L10,18L8.59,16.58Z" />
  </svg>
`;export{P as a,s as b,S as c,i as m,a as s};
